@@ -1,5 +1,9 @@
 #include "roaringbitmap.h"
 
+#ifdef PG_MODULE_MAGIC
+PG_MODULE_MAGIC;
+#endif
+
 /* Created by ZEROMAX on 2017/3/20.*/
 
 #define MAX_BITMAP_RANGE_END UINT64_C(0x100000000)
@@ -8,11 +12,6 @@
 
 /* GUC variables */
 
-typedef enum
-{
-    RBITMAP_OUTPUT_ARRAY,                /* output as int array */
-    RBITMAP_OUTPUT_BYTEA                 /* output as bytea */
-}RBITMAPOutputFormat;
 
 static const struct config_enum_entry output_format_options[] =
 {
@@ -21,7 +20,46 @@ static const struct config_enum_entry output_format_options[] =
     {NULL, 0, false}
 };
 
-static int    rbitmap_output_format = RBITMAP_OUTPUT_BYTEA;        /* output format */
+int    rbitmap_output_format = RBITMAP_OUTPUT_BYTEA;        /* output format */
+
+/* Malloc a buffer of size + alignment bytes and returns the aligned part.
+The offset between the real pointer and returned value was stored in p[-1].
+*/
+static inline void *pg_aligned_malloc(size_t alignment, size_t size) {
+    void *p;
+    void *porg;
+    assert(alignment <= 256);
+    porg = palloc(size + alignment);
+    p = (void *)((((size_t)porg + alignment) / alignment) * alignment);
+    *((unsigned char *)p-1) = (unsigned char)((size_t)p - (size_t)porg);
+    return p;
+}
+
+static inline void pg_aligned_free(void *memblock) {
+    void *porg;
+    if (memblock == NULL)
+        return;
+    porg = (void *)((size_t)memblock - *((unsigned char *)memblock-1));
+    if (porg == memblock)
+        porg = (void *)((size_t)porg - 256);
+    pfree(porg);
+}
+
+static inline void *pg_calloc(size_t nmemb, size_t size) {
+    return palloc0(nmemb * size);
+}
+
+static inline void pg_free(void *ptr) {
+    if (ptr == NULL) {
+        free(ptr);
+    } else {
+        pfree(ptr);
+    }
+}
+
+static inline void *pg_realloc(void *ptr, size_t size) {
+    return ptr == NULL ? palloc(size) : repalloc(ptr, size);
+}
 
 static roaring_memory_t rb_memory_hook = {
     .malloc = palloc,
@@ -226,7 +264,7 @@ roaringbitmap_in(PG_FUNCTION_ARGS) {
                 ptr++;
         }
 
-        /* Find the tail char '{' */
+        /* Find the tail char '}' */
         if (*ptr !='}'){
             roaring_bitmap_free(r1);
             ereport(ERROR,
@@ -375,14 +413,14 @@ rb_or_cardinality(PG_FUNCTION_ARGS) {
     bool ret;
 
     r1 = roaring_buffer_create(VARDATA(serializedbytes1),
-                               VARSIZE(serializedbytes1));
+                               VARSIZE(serializedbytes1) - VARHDRSZ);
     if (!r1)
         ereport(ERROR,
                 (errcode(ERRCODE_NULL_VALUE_NOT_ALLOWED),
                  errmsg("bitmap format is error")));
 
     r2 = roaring_buffer_create(VARDATA(serializedbytes2),
-                               VARSIZE(serializedbytes2));
+                               VARSIZE(serializedbytes2) - VARHDRSZ);
     if (!r2) {
         roaring_buffer_free(r1);
         ereport(ERROR,
@@ -416,14 +454,14 @@ rb_and(PG_FUNCTION_ARGS) {
     bytea *serializedbytes;
 
     r1 = roaring_buffer_create(VARDATA(serializedbytes1),
-                               VARSIZE(serializedbytes1));
+                               VARSIZE(serializedbytes1) - VARHDRSZ);
     if (!r1)
         ereport(ERROR,
                 (errcode(ERRCODE_NULL_VALUE_NOT_ALLOWED),
                  errmsg("bitmap format is error")));
 
     r2 = roaring_buffer_create(VARDATA(serializedbytes2),
-                               VARSIZE(serializedbytes2));
+                               VARSIZE(serializedbytes2) - VARHDRSZ);
     if (!r2) {
         roaring_buffer_free(r1);
         ereport(ERROR,
@@ -464,14 +502,14 @@ rb_and_cardinality(PG_FUNCTION_ARGS) {
     bool ret;
 
     r1 = roaring_buffer_create(VARDATA(serializedbytes1),
-                               VARSIZE(serializedbytes1));
+                               VARSIZE(serializedbytes1) - VARHDRSZ);
     if (!r1)
         ereport(ERROR,
                 (errcode(ERRCODE_NULL_VALUE_NOT_ALLOWED),
                  errmsg("bitmap format is error")));
 
     r2 = roaring_buffer_create(VARDATA(serializedbytes2),
-                               VARSIZE(serializedbytes2));
+                               VARSIZE(serializedbytes2) - VARHDRSZ);
     if (!r2) {
         roaring_buffer_free(r1);
         ereport(ERROR,
@@ -506,14 +544,14 @@ rb_andnot(PG_FUNCTION_ARGS) {
     bytea *serializedbytes;
 
     r1 = roaring_buffer_create(VARDATA(serializedbytes1),
-                               VARSIZE(serializedbytes1));
+                               VARSIZE(serializedbytes1) - VARHDRSZ);
     if (!r1)
         ereport(ERROR,
                 (errcode(ERRCODE_NULL_VALUE_NOT_ALLOWED),
                  errmsg("bitmap format is error")));
 
     r2 = roaring_buffer_create(VARDATA(serializedbytes2),
-                               VARSIZE(serializedbytes2));
+                               VARSIZE(serializedbytes2) - VARHDRSZ);
     if (!r2) {
         roaring_buffer_free(r1);
         ereport(ERROR,
@@ -554,14 +592,14 @@ rb_andnot_cardinality(PG_FUNCTION_ARGS) {
     bool ret;
 
     r1 = roaring_buffer_create(VARDATA(serializedbytes1),
-                               VARSIZE(serializedbytes1));
+                               VARSIZE(serializedbytes1) - VARHDRSZ);
     if (!r1)
         ereport(ERROR,
                 (errcode(ERRCODE_NULL_VALUE_NOT_ALLOWED),
                  errmsg("bitmap format is error")));
 
     r2 = roaring_buffer_create(VARDATA(serializedbytes2),
-                               VARSIZE(serializedbytes2));
+                               VARSIZE(serializedbytes2) - VARHDRSZ);
     if (!r2) {
         roaring_buffer_free(r1);
         ereport(ERROR,
@@ -634,14 +672,14 @@ rb_xor_cardinality(PG_FUNCTION_ARGS) {
     bool ret;
 
     r1 = roaring_buffer_create(VARDATA(serializedbytes1),
-                               VARSIZE(serializedbytes1));
+                               VARSIZE(serializedbytes1) - VARHDRSZ);
     if (!r1)
         ereport(ERROR,
                 (errcode(ERRCODE_NULL_VALUE_NOT_ALLOWED),
                  errmsg("bitmap format is error")));
 
     r2 = roaring_buffer_create(VARDATA(serializedbytes2),
-                               VARSIZE(serializedbytes2));
+                               VARSIZE(serializedbytes2) - VARHDRSZ);
     if (!r2) {
         roaring_buffer_free(r1);
         ereport(ERROR,
@@ -672,7 +710,7 @@ rb_cardinality(PG_FUNCTION_ARGS) {
     uint64 card;
 
     r1 = roaring_buffer_create(VARDATA(data),
-                               VARSIZE(data));
+                               VARSIZE(data) - VARHDRSZ);
     if (!r1)
         ereport(ERROR,
                 (errcode(ERRCODE_NULL_VALUE_NOT_ALLOWED),
@@ -696,7 +734,7 @@ rb_is_empty(PG_FUNCTION_ARGS) {
     bool isempty;
 
     r1 = roaring_buffer_create(VARDATA(data),
-                               VARSIZE(data));
+                               VARSIZE(data) - VARHDRSZ);
     if (!r1)
         ereport(ERROR,
                 (errcode(ERRCODE_NULL_VALUE_NOT_ALLOWED),
@@ -705,36 +743,36 @@ rb_is_empty(PG_FUNCTION_ARGS) {
     isempty = roaring_buffer_is_empty(r1);
     roaring_buffer_free(r1);
 
-    PG_RETURN_INT64(isempty);
+    PG_RETURN_BOOL(isempty);
 }
 
 //bitmap contains one value
-PG_FUNCTION_INFO_V1(rb_exsit);
-Datum rb_exsit(PG_FUNCTION_ARGS);
+PG_FUNCTION_INFO_V1(rb_exists);
+Datum rb_exists(PG_FUNCTION_ARGS);
 
 Datum
-rb_exsit(PG_FUNCTION_ARGS) {
+rb_exists(PG_FUNCTION_ARGS) {
     bytea *data = PG_GETARG_BYTEA_P(0);
     uint32 value = PG_GETARG_UINT32(1);
     roaring_buffer_t *r1;
-    bool isexsit;
+    bool isexist;
     bool ret;
 
     r1 = roaring_buffer_create(VARDATA(data),
-                               VARSIZE(data));
+                               VARSIZE(data) - VARHDRSZ);
     if (!r1)
         ereport(ERROR,
                 (errcode(ERRCODE_NULL_VALUE_NOT_ALLOWED),
                  errmsg("bitmap format is error")));
 
-    ret = roaring_buffer_contains(r1, value, &isexsit);
+    ret = roaring_buffer_contains(r1, value, &isexist);
     roaring_buffer_free(r1);
     if(!ret)
         ereport(ERROR,
                 (errcode(ERRCODE_NULL_VALUE_NOT_ALLOWED),
                  errmsg("bitmap format is error")));
 
-    PG_RETURN_BOOL(isexsit);
+    PG_RETURN_BOOL(isexist);
 }
 
 //bitmap equals
@@ -751,14 +789,14 @@ rb_equals(PG_FUNCTION_ARGS) {
     bool ret;
 
     r1 = roaring_buffer_create(VARDATA(serializedbytes1),
-                               VARSIZE(serializedbytes1));
+                               VARSIZE(serializedbytes1) - VARHDRSZ);
     if (!r1)
         ereport(ERROR,
                 (errcode(ERRCODE_NULL_VALUE_NOT_ALLOWED),
                  errmsg("bitmap format is error")));
 
     r2 = roaring_buffer_create(VARDATA(serializedbytes2),
-                               VARSIZE(serializedbytes2));
+                               VARSIZE(serializedbytes2) - VARHDRSZ);
     if (!r2) {
         roaring_buffer_free(r1);
         ereport(ERROR,
@@ -791,14 +829,14 @@ rb_not_equals(PG_FUNCTION_ARGS) {
     bool ret;
 
     r1 = roaring_buffer_create(VARDATA(serializedbytes1),
-                               VARSIZE(serializedbytes1));
+                               VARSIZE(serializedbytes1) - VARHDRSZ);
     if (!r1)
         ereport(ERROR,
                 (errcode(ERRCODE_NULL_VALUE_NOT_ALLOWED),
                  errmsg("bitmap format is error")));
 
     r2 = roaring_buffer_create(VARDATA(serializedbytes2),
-                               VARSIZE(serializedbytes2));
+                               VARSIZE(serializedbytes2) - VARHDRSZ);
     if (!r2) {
         roaring_buffer_free(r1);
         ereport(ERROR,
@@ -831,14 +869,14 @@ rb_intersect(PG_FUNCTION_ARGS) {
     bool ret;
 
     r1 = roaring_buffer_create(VARDATA(serializedbytes1),
-                               VARSIZE(serializedbytes1));
+                               VARSIZE(serializedbytes1) - VARHDRSZ);
     if (!r1)
         ereport(ERROR,
                 (errcode(ERRCODE_NULL_VALUE_NOT_ALLOWED),
                  errmsg("bitmap format is error")));
 
     r2 = roaring_buffer_create(VARDATA(serializedbytes2),
-                               VARSIZE(serializedbytes2));
+                               VARSIZE(serializedbytes2) - VARHDRSZ);
     if (!r2) {
         roaring_buffer_free(r1);
         ereport(ERROR,
@@ -871,14 +909,14 @@ rb_contains(PG_FUNCTION_ARGS) {
     bool ret;
 
     r1 = roaring_buffer_create(VARDATA(serializedbytes1),
-                               VARSIZE(serializedbytes1));
+                               VARSIZE(serializedbytes1) - VARHDRSZ);
     if (!r1)
         ereport(ERROR,
                 (errcode(ERRCODE_NULL_VALUE_NOT_ALLOWED),
                  errmsg("bitmap format is error")));
 
     r2 = roaring_buffer_create(VARDATA(serializedbytes2),
-                               VARSIZE(serializedbytes2));
+                               VARSIZE(serializedbytes2) - VARHDRSZ);
     if (!r2) {
         roaring_buffer_free(r1);
         ereport(ERROR,
@@ -911,14 +949,14 @@ rb_containedby(PG_FUNCTION_ARGS) {
     bool ret;
 
     r1 = roaring_buffer_create(VARDATA(serializedbytes1),
-                               VARSIZE(serializedbytes1));
+                               VARSIZE(serializedbytes1) - VARHDRSZ);
     if (!r1)
         ereport(ERROR,
                 (errcode(ERRCODE_NULL_VALUE_NOT_ALLOWED),
                  errmsg("bitmap format is error")));
 
     r2 = roaring_buffer_create(VARDATA(serializedbytes2),
-                               VARSIZE(serializedbytes2));
+                               VARSIZE(serializedbytes2) - VARHDRSZ);
     if (!r2) {
         roaring_buffer_free(r1);
         ereport(ERROR,
@@ -951,14 +989,14 @@ rb_jaccard_dist(PG_FUNCTION_ARGS) {
     bool ret;
 
     r1 = roaring_buffer_create(VARDATA(serializedbytes1),
-                               VARSIZE(serializedbytes1));
+                               VARSIZE(serializedbytes1) - VARHDRSZ);
     if (!r1)
         ereport(ERROR,
                 (errcode(ERRCODE_NULL_VALUE_NOT_ALLOWED),
                  errmsg("bitmap format is error")));
 
     r2 = roaring_buffer_create(VARDATA(serializedbytes2),
-                               VARSIZE(serializedbytes2));
+                               VARSIZE(serializedbytes2) - VARHDRSZ);
     if (!r2) {
         roaring_buffer_free(r1);
         ereport(ERROR,
@@ -1045,7 +1083,7 @@ rb_min(PG_FUNCTION_ARGS) {
     bool ret;
 
     r1 = roaring_buffer_create(VARDATA(data),
-                               VARSIZE(data));
+                               VARSIZE(data) - VARHDRSZ);
     if (!r1)
         ereport(ERROR,
                 (errcode(ERRCODE_NULL_VALUE_NOT_ALLOWED),
@@ -1080,7 +1118,7 @@ rb_max(PG_FUNCTION_ARGS) {
     bool ret;
 
     r1 = roaring_buffer_create(VARDATA(data),
-                               VARSIZE(data));
+                               VARSIZE(data) - VARHDRSZ);
     if (!r1)
         ereport(ERROR,
                 (errcode(ERRCODE_NULL_VALUE_NOT_ALLOWED),
@@ -1115,7 +1153,7 @@ rb_rank(PG_FUNCTION_ARGS) {
     bool ret;
 
     r1 = roaring_buffer_create(VARDATA(data),
-                               VARSIZE(data));
+                               VARSIZE(data) - VARHDRSZ);
     if (!r1)
         ereport(ERROR,
                 (errcode(ERRCODE_NULL_VALUE_NOT_ALLOWED),
@@ -1142,16 +1180,16 @@ rb_index(PG_FUNCTION_ARGS) {
     roaring_buffer_t *r1;
     uint64 rank;
     int64 result;
-    bool ret,isexsit;
+    bool ret,isexist;
 
     r1 = roaring_buffer_create(VARDATA(data),
-                               VARSIZE(data));
+                               VARSIZE(data) - VARHDRSZ);
     if (!r1)
         ereport(ERROR,
                 (errcode(ERRCODE_NULL_VALUE_NOT_ALLOWED),
                  errmsg("bitmap format is error")));
 
-    ret = roaring_buffer_contains(r1, value, &isexsit);
+    ret = roaring_buffer_contains(r1, value, &isexist);
     if(!ret)
     {
         roaring_buffer_free(r1);
@@ -1161,7 +1199,7 @@ rb_index(PG_FUNCTION_ARGS) {
     }
 
     result = -1;
-    if(isexsit)
+    if(isexist)
     {
         ret = roaring_buffer_rank(r1, value, &rank);
         roaring_buffer_free(r1);
@@ -1186,7 +1224,6 @@ rb_fill(PG_FUNCTION_ARGS) {
     int64 rangestart = PG_GETARG_INT64(1);
     int64 rangeend = PG_GETARG_INT64(2);
     roaring_bitmap_t *r1;
-    roaring_bitmap_t *r2;
     size_t expectedsize;
     bytea *serializedbytes;
 
@@ -1205,15 +1242,7 @@ rb_fill(PG_FUNCTION_ARGS) {
                  errmsg("bitmap format is error")));
 
     if (rangestart < rangeend) {
-        r2 = roaring_bitmap_from_range(rangestart, rangeend, 1);
-        if (!r2) {
-            roaring_bitmap_free(r1);
-            ereport(ERROR,
-                    (errcode(ERRCODE_NULL_VALUE_NOT_ALLOWED),
-                     errmsg("failed to create bitmap")));
-        }
-        roaring_bitmap_or_inplace(r1, r2);
-        roaring_bitmap_free(r2);
+        roaring_bitmap_add_range(r1,rangestart, rangeend);
     }
 
     expectedsize = roaring_bitmap_portable_size_in_bytes(r1);
@@ -1235,7 +1264,6 @@ rb_clear(PG_FUNCTION_ARGS) {
     int64 rangestart = PG_GETARG_INT64(1);
     int64 rangeend = PG_GETARG_INT64(2);
     roaring_bitmap_t *r1;
-    roaring_bitmap_t *r2;
     size_t expectedsize;
     bytea *serializedbytes;
 
@@ -1254,16 +1282,7 @@ rb_clear(PG_FUNCTION_ARGS) {
                  errmsg("bitmap format is error")));
 
     if (rangestart < rangeend) {
-        r2 = roaring_bitmap_from_range(rangestart, rangeend, 1);
-        if (!r2) {
-            roaring_bitmap_free(r1);
-            ereport(ERROR,
-                    (errcode(ERRCODE_NULL_VALUE_NOT_ALLOWED),
-                     errmsg("failed to create bitmap")));
-        }
-
-        roaring_bitmap_andnot_inplace(r1, r2);
-        roaring_bitmap_free(r2);
+        roaring_bitmap_remove_range(r1,rangestart, rangeend);
     }
 
     expectedsize = roaring_bitmap_portable_size_in_bytes(r1);
@@ -1587,7 +1606,7 @@ rb_build(PG_FUNCTION_ARGS) {
     CHECKARRVALID(a);
 
     na = ARRNELEMS(a);
-    da = ARRPTR(a);
+    da = (int *)ARRPTR(a);
 
     r1 = roaring_bitmap_create();
 
